;==============================================================================
;       Test Department's WINDOWS 32 BIT x86 ASSEMBLY EXAMPLE's         050
;==============================================================================

;==============================================================================
; ==> Part 050 : windows high resolution timer
;------------------------------------------------------------------------------
; Greetings from germany,
; the idea here is to use a HIGH RESOLUTION TIMER from the multimedia library.
; This timer runs in its own thread !
; API "timeSetEvent" / "timeKillEvent" (Mmedia.hlp) is the function call.
; This timer is much more exact than a standard windows timer.
; This timer could only beat by multimedia applications using MCI services.
; In the window procedure (WP1) reacting to a WM_CREATE message I setup a
; standard timer (TP0) and the discussed high resolution timer (TP1).
; This timer calls a timer procedure (TP1) on the specified delay time.
; For example I include a standard windows timer to visualize the difference.
; For Fun you can also use an Atomic Radio Clock to check the accuracy ...

;==============================================================================
; Assembler directives
;------------------------------------------------------------------------------
.386                    ; specifies the processor our program want run on
.Model Flat ,StdCall    ; Flat for Win9x (32 Bit), Calling Convention
option casemap:none     ; case sensitive !!!

;==============================================================================
; Include all files where API functins resist you want use, set correct path
;------------------------------------------------------------------------------
include D:\Masm32\include\windows.inc
includelib kernel32.lib
includelib user32.lib
includelib winmm.lib

;==============================================================================
; Declaration of used API functions,take a look into WIN32.HLP and *.inc files
;------------------------------------------------------------------------------
GetModuleHandleA        PROTO :DWORD
LoadIconA               PROTO :DWORD,:DWORD
LoadCursorA             PROTO :DWORD,:DWORD
RegisterClassExA        PROTO :DWORD
CreateWindowExA         PROTO :DWORD,:DWORD,:DWORD,:DWORD,:DWORD,:DWORD,:DWORD,
                              :DWORD,:DWORD,:DWORD,:DWORD,:DWORD
ShowWindow              PROTO :DWORD,:DWORD
UpdateWindow            PROTO :DWORD
GetMessageA             PROTO :DWORD,:DWORD,:DWORD,:DWORD
TranslateMessage        PROTO :DWORD
DispatchMessageA        PROTO :DWORD
PostQuitMessage         PROTO :DWORD
DefWindowProcA          PROTO :DWORD,:DWORD,:DWORD,:DWORD
ExitProcess             PROTO :DWORD
DestroyWindow           PROTO :DWORD
SetWindowTextA          PROTO :DWORD,:DWORD
SetTimer                PROTO :DWORD,:DWORD,:DWORD,:DWORD
KillTimer               PROTO :DWORD,:DWORD
timeGetDevCaps          PROTO :DWORD,:DWORD
timeSetEvent            PROTO :DWORD,:DWORD,:DWORD,:DWORD,:DWORD
timeKillEvent           PROTO :DWORD

;==============================================================================
; .const    = the constants area starts here,constants are defined and fixed
;------------------------------------------------------------------------------
.const
; - Parameter MAIN WINDOW CallBack Procedure ( API=RegisterClassExA ) -
WP1_CallBack            equ [ebp+4]     ;return address
WP1_hWnd                equ [ebp+8]     ;handle of window who receives message
WP1_uMsg                equ [ebp+12]    ;the message number
WP1_wParam              equ [ebp+16]    ;extra info about the message
WP1_lParam              equ [ebp+20]    ;extra info about the message

;==============================================================================
; .Data         = the data area starts here, datas are defined but not fixed
;------------------------------------------------------------------------------
.Data
IconName                db "TDIcon",0               ;icon name in rc file
MenuName                db "TDMenu",0               ;menu name in rc file
ClassName               db "TDWinClass",0           ;name of windows class
WindowName              db "Test Department - http://surf.to/TestD",0;winname
STATIC_WindowClass      db "STATIC",0               ;name of windows class
STATIC_STextWindowName  db "Windows Standard Timer in seconds :",0;
STATIC_STimeWindowName  db "000",0                  ;
STimeupdate             db "000",0                  ;
STATIC_PTextWindowName  db "Windows Precision Timer in seconds :",0;
STATIC_PTimeWindowName  db "000",0                  ;
PTimeupdate             db "000",0                  ;

;==============================================================================
; .Data?    = the data? area starts here, not defined and not fixed
;------------------------------------------------------------------------------
.data?
STATIC_STexthWnd    dd ?            ;handle STATIC, standard text window
STATIC_STimehWnd    dd ?            ;handle STATIC, standard time window
STATIC_PTexthWnd    dd ?            ;handle STATIC, precision text window
STATIC_PTimehWnd    dd ?            ;handle STATIC, precision time window
timerevent1         dd ?            ;handle high precision timer

align 4
; - WndClassEx Structure ( API=RegisterClassExA ) -
cbSize              dd ?            ;size in bytes of this structure
style               dd ?            ;window style
lpfnWndProc         dd ?            ;address of user proc function
cbclsExtra          dd ?            ;extra bytes to allocate set to 0
cbWndExtra          dd ?            ;extra bytes class directive, rc file
hInstance           dd ?            ;program handle(API=GetModuleHandleA)
hIcon               dd ?            ;handle of icon (API=LoadIconA)
hcursor             dd ?            ;handle of cursor (API=LoadCursor)
hbrBackground       dd ?            ;background color, 0=transparent
lpszMenuName        dd ?            ;name of menu class in resource file
lpszClassName       dd ?            ;name of windows this window class
hIconSm             dd ?            ;iconhandle 0=search in resource file

align 4
; - Msg Structure ( API=GetMessageA ) - member POINT = POINT structure
hWnd                dd ?            ;handle of window who receives message
message             dd ?            ;the message number
wParam              dd ?            ;extra info about the message
lParam              dd ?            ;extra info about the message 
time                dd ?            ;time the message was posted 
xpt                 dd ?            ;cursor x-position, POINT struc
ypt                 dd ?            ;cursor x-position, POINT struc 

align 4
; - TIMECAPS Structure ( API=timeGetDevCaps ) -
wPeriodMin          dd ?            ;TIMER minimum supported resolution
wPeriodMax          dd ?            ;TIMER maximum supported resolution

;==============================================================================
; .CODE = our code area starts here     Main = label of our program code
;------------------------------------------------------------------------------
.Code
Main:

;==============================================================================
; Always get your program ID first (API=GetModuleHandleA)
;------------------------------------------------------------------------------
push    0h                          ;lpModuleHandle, 0=get program handle
call    GetModuleHandleA            ;- API Function -
mov     hInstance,eax               ;return value in eax=handle of program

;==============================================================================
; The API function "RegisterClassExA" registers a window class.
; This API needs a "WNDCLASSEX" structure so we fill it with correct values.
;------------------------------------------------------------------------------
mov     cbSize,30h                  ;size in bytes of WNDCLASSEX structure
mov     style,3h                    ;window style
mov     lpfnWndProc,OFFSET WP1      ;address of user lpfnWndProc function
mov     cbclsExtra,0h               ;extra bytes to allocate set to 0
mov     cbWndExtra,0h               ;class directive in rc file
mov     hbrBackground,2h            ;background,1=background(parameter+1)
mov     lpszMenuName,0h             ;menu name in resource file,0=no menu
mov     lpszClassName,OFFSET ClassName;name of windows class
mov     hIconSm,0h                  ;iconhandle 0=search in rc file
;------------------------------------------------------------------------------
; API "LoadIconA" loads an icon defined in the resource file and stores the
; handle in the "WNDCLASSEX" structure
;------------------------------------------------------------------------------
push    OFFSET IconName             ;icon-string or icon resource id
push    hInstance                   ;our program handle
call    LoadIconA                   ;- API Function -
mov     hIcon,eax                   ;store handle of newly loaded icon
;------------------------------------------------------------------------------
; API "LoadCursorA" loads a default system cursor, in this case we must set
; hInstance to 0 and lpCursorName to a default system cursor value, here 32512
; Then we store the cursor handle in the "WNDCLASSEX" structure
;------------------------------------------------------------------------------
push    32512                       ;lpCursorName, default value in dezimal
push    0h                          ;hInstance, 0=default system cursor
call    LoadCursorA                 ;- API Function -
mov     hcursor,eax                 ;store handle of the cursor
;------------------------------------------------------------------------------
; Now, after filled the "WNDCLASSEX" structure we call API "RegisterClassEx"
;------------------------------------------------------------------------------
push    OFFSET cbSize               ;pointer to WNDCLASSEX structure
call    RegisterClassExA            ;- API Function -

;==============================================================================
; API "CreateWindowExA" creates an overlapped, pop-up, or child window with an
; extended style. The return value in EAX is the handle of the new window.
; This API sends a WM_CREATE message to the window procedure (WP1).
;------------------------------------------------------------------------------
push    0h                          ;lpParam, extra pointer data 0=no data
push    hInstance                   ;hInstance, handle of our program
push    0h                          ;hMenu, handle window menu 0=class menu
push    0h                          ;hWndParent, handle parent window 0=no
push    00000060h                   ;intnHeight, window height pixel
push    00000140h                   ;intnWidth, window width pixel
push    00000010h                   ;inty, vertical position window
push    00000090h                   ;intx, horizontal position window
push    04CA0000h                   ;dwStyle, look into WIN32.HLP
push    OFFSET WindowName           ;lpWindowName, pointer to window name
push    OFFSET ClassName            ;lpClassName, pointer to class name
push    0300h                       ;dwExStyle, extra window style 0=no
call    CreateWindowExA             ;- API Function -
mov     hWnd,eax                    ;hwnd,return value=handle of window

;==============================================================================
; API "ShowWindow" function sets the specified window's show state. 
;------------------------------------------------------------------------------
push    1h                          ;nCmdShow, show state 1=SW_SHOWNORMAL
push    hWnd                        ;hwnd, handle of window
call    ShowWindow                  ;- API Function -

;==============================================================================
; API "UpdateWindow" updates the area of the specified window by sending a
; WM_PAINT message to the window if the window's update region is not empty.
;------------------------------------------------------------------------------
push    hWnd                        ;hwnd, handle of window
call    UpdateWindow                ;- API Function -

LoopGetMessage:
;==============================================================================
; API "GetMessageA" retrieves a message + places it in the specified structure.
;------------------------------------------------------------------------------
push    0h                          ;wMsgFilterMax, highest message value
push    0h                          ;wMsgFilterMin, lowest message value
push    0h                          ;hWnd, handle of window who gets msg.
push    OFFSET hWnd                 ;lpMsg, pointer to MSG structure
call    GetMessageA                 ;- API Function -
cmp     eax,0h                      ;check if return value=0 (exit)
je      ExitPrg                     ;if return value is 0 goto LABEL

;==============================================================================
; API "TranslateMessage" translates virtual-key messages in character messages
;------------------------------------------------------------------------------
push    OFFSET hWnd                 ;lpMSG, pointer to msg structure
call    TranslateMessage            ;- API Function - keyboard code

;==============================================================================
; API "DispatchMessageA" function dispatches a message to a window procedure.
;------------------------------------------------------------------------------
push    OFFSET hWnd                 ;lpMSG, pointer to msg structure
call    DispatchMessageA            ;- API Function -
jmp     LoopGetMessage              ;check for message again, goto LABEL

ExitPrg:
;==============================================================================
; Next we terminate our program (API=ExitProcess)
;------------------------------------------------------------------------------
push    hInstance                   ;push our programm handle to exit
call    ExitProcess                 ;- API Function -

;##############################################################################
; The Window Procedure (API=RegisterClassExA) for this registered window.
;------------------------------------------------------------------------------
WP1:
push    ebp                         ;create stack frame
mov     ebp,esp                     ;
pushad                              ;push all register to the stack

mov     eax,WP1_uMsg                ;move the message number to eax
;==============================================================================
; WM_CREATE (value=01h) message received ?
;------------------------------------------------------------------------------
WP1_uMsg_01h:
cmp     eax,1h                      ;check if WM_CREATE message recieved
jne     WP1_uMsg_02h                ;if not goto LABEL
;------------------------------------------------------------------------------
; API "CreateWindowExA" creates a window with a predefined class name (STATIC).
; The return value in EAX is the handle of the new window.
;------------------------------------------------------------------------------
push    0h                          ;lpParam, extra pointer data 0=no data
push    hInstance                   ;hInstance, handle of our program
push    0h                          ;hMenu, handle window menu 0=class menu
push    WP1_hWnd                    ;hWndParent, handle parent window 0=no
push    00000016h                   ;intnHeight, window height pixel
push    00000100h                   ;intnWidth, window width pixel
push    00000008h                   ;inty, vertical position window
push    00000008h                   ;intx, horizontal position window
push    54000001h                   ;dwStyle, look WIN32.HLP + windows.inc
push    OFFSET STATIC_STextWindowName;lpWindowName, pointer to window name
push    OFFSET STATIC_WindowClass   ;lpClassName, pointer to class name
push    1h                          ;dwExStyle,look WIN32.HLP + windows.inc
call    CreateWindowExA             ;- API Function -
mov     STATIC_STexthWnd,eax        ;hwnd,return value=handle of window

push    0h                          ;lpParam, extra pointer data 0=no data
push    hInstance                   ;hInstance, handle of our program
push    0h                          ;hMenu, handle window menu 0=class menu
push    WP1_hWnd                    ;hWndParent, handle parent window 0=no
push    00000016h                   ;intnHeight, window height pixel
push    00000020h                   ;intnWidth, window width pixel
push    00000008h                   ;inty, vertical position window
push    00000110h                   ;intx, horizontal position window
push    54000001h                   ;dwStyle, look WIN32.HLP + windows.inc
push    OFFSET STATIC_STimeWindowName;lpWindowName, pointer to window name
push    OFFSET STATIC_WindowClass   ;lpClassName, pointer to class name
push    1h                          ;dwExStyle,look WIN32.HLP + windows.inc
call    CreateWindowExA             ;- API Function -
mov     STATIC_STimehWnd,eax        ;hwnd,return value=handle of window

push    0h                          ;lpParam, extra pointer data 0=no data
push    hInstance                   ;hInstance, handle of our program
push    0h                          ;hMenu, handle window menu 0=class menu
push    WP1_hWnd                    ;hWndParent, handle parent window 0=no
push    00000016h                   ;intnHeight, window height pixel
push    00000100h                   ;intnWidth, window width pixel
push    00000020h                   ;inty, vertical position window
push    00000008h                   ;intx, horizontal position window
push    54000001h                   ;dwStyle, look WIN32.HLP + windows.inc
push    OFFSET STATIC_PTextWindowName;lpWindowName, pointer to window name
push    OFFSET STATIC_WindowClass   ;lpClassName, pointer to class name
push    1h                          ;dwExStyle,look WIN32.HLP + windows.inc
call    CreateWindowExA             ;- API Function -
mov     STATIC_PTexthWnd,eax        ;hwnd,return value=handle of window

push    0h                          ;lpParam, extra pointer data 0=no data
push    hInstance                   ;hInstance, handle of our program
push    0h                          ;hMenu, handle window menu 0=class menu
push    WP1_hWnd                    ;hWndParent, handle parent window 0=no
push    00000016h                   ;intnHeight, window height pixel
push    00000020h                   ;intnWidth, window width pixel
push    00000020h                   ;inty, vertical position window
push    00000110h                   ;intx, horizontal position window
push    54000001h                   ;dwStyle, look WIN32.HLP + windows.inc
push    OFFSET STATIC_PTimeWindowName;lpWindowName, pointer to window name
push    OFFSET STATIC_WindowClass   ;lpClassName, pointer to class name
push    1h                          ;dwExStyle,look WIN32.HLP + windows.inc
call    CreateWindowExA             ;- API Function -
mov     STATIC_PTimehWnd,eax        ;hwnd,return value=handle of window
;------------------------------------------------------------------------------
; timeGetDevCaps function queries the timer device to determine its resolution.
; Returned values in the TIMECAPS struc are different from system to system.
;------------------------------------------------------------------------------
push    8h                          ;cbtc, size in bytes TIMECAPS structure
push    OFFSET wPeriodMin           ;ptc, address of a TIMECAPS structure
call    timeGetDevCaps              ;- API Function -
;------------------------------------------------------------------------------
; The timeSetEvent function starts a PRECISION timer event. After activated, it
; calls the specified callback function. It runs in its own thread ! 
;------------------------------------------------------------------------------
push    1h                          ;fuEvent, timer type, TIME_PERIODIC=1h
push    0h                          ;dwUser, user-supplied callback data 
push    OFFSET TP1                  ;lpTimeProc, address callback function
push    wPeriodMin                  ;uResolution, resolution in ms
push    1000                        ;uDelay, event delay in milliseconds
call    timeSetEvent                ;- API Function -
mov     timerevent1,eax             ;store handle
;------------------------------------------------------------------------------
; API "SetTimer" creates a STANDARD timer with the specified time-out value.
;------------------------------------------------------------------------------
push    OFFSET TP0                  ;tmprc, address timer procedure
push    1000                        ;uTimeout, time-out value milliseconds
push    10h                         ;idTimer, timer identifier, nonzero
push    WP1_hWnd                    ;hwnd, handle of window for timer msg.
call    SetTimer                    ;- API Function -
jmp     WP1_return                  ;

;==============================================================================
; WM_DESTROY (value=02h) message received ?
;------------------------------------------------------------------------------
WP1_uMsg_02h:
cmp     eax,2h                      ;check if value=2h (WM_DESTROY)
jne     WP1_uMsg_112h               ;if not 2h go to LABEL
call    My_CleanSystem              ;- SubRoutine -
;------------------------------------------------------------------------------
; API "PostQuitMessage" indicates to Windows a request to terminate
;------------------------------------------------------------------------------
push    0h                          ;nExitCode, exit code=wParam
call    PostQuitMessage             ;- API Function -
popad                               ;pop all register back from stack
xor     eax,eax                     ;set eax to 0 to exit our program
mov     esp,ebp                     ;delete stack frame
pop     ebp                         ;
ret     10h                         ;return and clear stack

;==============================================================================
; WM_SYSCOMMAND (value=112h) message recieved ?
;------------------------------------------------------------------------------
WP1_uMsg_112h:
cmp     eax,112h                    ;check if WM_COMMAND message recieved
jne     WP1_return                  ;if not goto label
mov     eax,WP1_wParam              ;extra info about the message
cmp     eax,0F060h                  ;SC_CLOSE=0F060h received ?
jne     WP1_return                  ;
call    My_CleanSystem              ;- SubRoutine -
jmp     WP1_return

;==============================================================================
; API "DefWindowProcA" calls the window procedure to provide default processing
; for any window messages that an application does not process.
; This function ensures that every message is processed.
; It is called with the same parameters received by the window procedure.
;------------------------------------------------------------------------------
WP1_return:
popad                               ;pop all register from stack
push    WP1_lParam                  ;extra info about the message
push    WP1_wParam                  ;extra info about the message
push    WP1_uMsg                    ;the message number
push    WP1_hWnd                    ;handle of window who receives message
call    DefWindowProcA              ;- API Function -
mov     esp,ebp                     ;delete stack frame
pop     ebp                         ;
ret     10h                         ;return and clear stack
;##############################################################################

;##############################################################################
; STANDARD TIMER procedure (Win32.hlp) !
;------------------------------------------------------------------------------
TP0:
pushad                              ;push all register to the stack
mov     ebx,OFFSET STimeupdate      ;update standard timer string
mov     ah,30h
mov     al,[ebx+2]
inc     al
cmp     al,39h
ja      Sdigit1
mov     [ebx+2],al
jmp     Out_TP0
Sdigit1:
mov     [ebx+2],ah
mov     al,[ebx+1]
inc     al
cmp     al,39h
ja      Sdigit2
mov     [ebx+1],al
jmp     Out_TP0
Sdigit2:
mov     [ebx+1],ah
mov     al,[ebx]
inc     al
cmp     al,39h
ja      SdigitReset
mov     [ebx],al
jmp     Out_TP0
SdigitReset:
mov     [ebx+2],ah
mov     [ebx+1],ah
mov     [ebx],ah
Out_TP0:
;------------------------------------------------------------------------------
; API "SetWindowTextA" set's the text of the specified window's title bar.
;------------------------------------------------------------------------------
push    OFFSET STimeupdate          ;lpsz, address of string
push    STATIC_STimehWnd            ;hwnd, handle of window or control
call    SetWindowTextA              ;- API Function -
TP0_return:
popad                               ;pop all register back from stack
ret     10h                         ;return and clear stack
;##############################################################################

;##############################################################################
; HIGH RESOLUTION TIMER procedure (Mmedia.hlp) !
;------------------------------------------------------------------------------
TP1:
pushad                              ;push all register to the stack
mov     ebx,OFFSET PTimeupdate      ;update precision timer string
mov     ah,30h
mov     al,[ebx+2]
inc     al
cmp     al,39h
ja      Pdigit1
mov     [ebx+2],al
jmp     Out_TP1
Pdigit1:
mov     [ebx+2],ah
mov     al,[ebx+1]
inc     al
cmp     al,39h
ja      Pdigit2
mov     [ebx+1],al
jmp     Out_TP1
Pdigit2:
mov     [ebx+1],ah
mov     al,[ebx]
inc     al
cmp     al,39h
ja      PdigitReset
mov     [ebx],al
jmp     Out_TP1
PdigitReset:
mov     [ebx+2],ah
mov     [ebx+1],ah
mov     [ebx],ah
Out_TP1:
;------------------------------------------------------------------------------
; API "SetWindowTextA" set's the text of the specified window's title bar.
;------------------------------------------------------------------------------
push    OFFSET PTimeupdate          ;lpsz, address of string
push    STATIC_PTimehWnd            ;hwnd, handle of window or control
call    SetWindowTextA              ;- API Function -
TP1_return:
popad                               ;pop all register back from stack
ret     14h                         ;return and clear stack (14h !!!)
;##############################################################################

;******************************************************************************
; My own subroutine(s) for a compacter code resist here ...
;------------------------------------------------------------------------------
My_CleanSystem:
;------------------------------------------------------------------------------
; The timeKillEvent function cancels a specified high precision timer event.
;------------------------------------------------------------------------------
push    timerevent1                 ;uTimerID, ID of timer event to cancel
call    timeKillEvent               ;- API Function -
;------------------------------------------------------------------------------
; API "KillTimer" destroys the specified standard timer.
;------------------------------------------------------------------------------
push    10h                         ;uIDEvent, timer identifier
push    WP1_hWnd                    ;hWnd, handle of window installed timer
call    KillTimer                   ;- API Function -
ret
;******************************************************************************

;==============================================================================
; end Main = end of our program code
;------------------------------------------------------------------------------
end Main                            ;end of our program code, entry point

;==============================================================================
; To create the exe file use this commands with your Microsoft Assembler/Linker
;------------------------------------------------------------------------------
; ml.exe /c /coff td_win32asm_050.asm                       ;asm command
; rc.exe /v rsrc.rc                                         ;rc command
; cvtres.exe /machine:ix86 rsrc.res
; link.exe /subsystem:windows td_win32asm_050.obj rsrc.obj  ;link command
;==============================================================================